
=head1 NAME

Zonk::Pronk -- blorpoesu

=head1 DESCRIPTION

This is just a test file.

This is a test Pod document in Latin-1. Its content is the last two
paragraphs of Baudelaire's I<Le Joujou du pauvre>.

A travers ces barreaux symboliques séparant deux mondes, la grande route
et le château, l'enfant pauvre montrait à l'enfant riche son propre
joujou, que celui-ci examinait avidement comme un objet rare et inconnu.
Or, ce joujou, que le petit souillon agaçait, agitait et secouait dans
une boîte grillée, c'était un rat vivantE<160>! Les parents, par économie
sans doute, avaient tiré le joujou de la vie elle-même. 

Et les deux enfants se riaient l'un à l'autre fraternellement, avec des
dents d'une I<égale> blancheur.

=head2 As Verbatim

 A travers ces barreaux symboliques séparant deux mondes, la grande route
 et le château, l'enfant pauvre montrait à l'enfant riche son propre
 joujou, que celui-ci examinait avidement comme un objet rare et inconnu.
 Or, ce joujou, que le petit souillon agaçait, agitait et secouait dans
 une boîte grillée, c'était un rat vivant !  Les parents, par économie
 sans doute, avaient tiré le joujou de la vie elle-même. 

 Et les deux enfants se riaient l'un à l'autre fraternellement, avec des
 dents d'une égale blancheur.

[end]

=cut



print "HOOBOY!\n";
1;

